/***************************************************
* C source with CCS
*  File Name: 20230731_K454_newMK301B_1822.c
 * rev 1.0 clock, moving average
 * rev 2.0 adjust length to detect, 20171119
 * rev 3.0 get less short range automatically , 20171202
 * rev 4.0 get fixed least range automatically , 20171206
 * rev 5.0 get fixed least range automatically , 20230731 
*  Description: 
*  PIN A0 for pot
*  PIN A1 for relay
*  PIN A2 for trigger
*  PIN A3 for echo
*  PIN A4 for osc
*  PIN A5 for osc
 * COPYRIGHT 2023 MYCOMKITS.COM, owned by CNET LIMITED
***************************************************/
// include headre file
#include	<12f1822.h>
#DEVICE ADC=10
// settings
#fuses	HS, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT, NODEBUG
#fuses	NOCLKOUT, NOWRT, PLL_SW, NOSTVREN
#use delay(CLOCK = 4000000)
//
#use	fast_io(a)
//
#define Amode 0x39			//
// HC-SR04 distance sensor, two signal
#define trig PIN_A2
#define echo PIN_A3
#define relay PIN_A1
// variables
float numberf = 0;                 // -20 to +120, 
float coeff = 0.034;// (340)/10000
//long  hyste =0;
float set_value_f;
int i;
float value_s[8];
int count=1;
long set_value;
long min_value;
int error_flag=0;
long adjust=10; //
// prototyping
#separate
void initializing();
#separate
void get_pot();
#separate
void hcsr04_read();
#separate
void hcsr04_read_1st();
#separate
void relay_2time(); //for test
// main function
void main()
{
	initializing();
	// main loop
	while(1)
	{
        get_pot();	// get value from potentiometer
        //
        if(input(relay) == 1)
        {
            delay_ms(3000);   //
        }    
        else delay_ms(100);   //min 60ms
        hcsr04_read(); 	// get distance from HCSR04
        //
        if(numberf < ( min_value - set_value - adjust))   // set_value is min_value+%ofsetvalue
        {
            output_high(relay);
        }
        else if(input(relay) == 1)
        {
            output_low(relay);	// relay off
        }
	}
}
//
#separate
void initializing()
{
	set_tris_a(Amode);	//
	// A/D converter initialize AN0
    setup_adc_ports(sAN0 | VSS_VDD);	// gnd to 5v
    setup_adc(ADC_CLOCK_DIV_4);			// 2usec
	// initialize timer 1 for counting distance
	setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);	// 1us cycle
	set_timer1(0x0);             //
	//
    output_low(relay);	// relay off
    //
    delay_ms(1000); //delay for stable
	get_pot();	// get value from potentiometer
    value_s[0] =6800;  //set max value
    count = 30;    //count 10 sec
    while(count > 0)
    {
        hcsr04_read_1st();	// get distance, mm unit, 0 to 6800
        count = count - 1;
        delay_ms(100);  //total 10seconds
    }    //
    min_value = value_s[0];
    //
}
//
#separate
void get_pot()
{
	set_adc_channel(0);	//must wait 65u, AN8
	delay_us(100);
	set_value = read_adc();	// 0 to 1023, 0V to 1.024V
    //
    if(set_value < 1023) set_value = 1023 - set_value;
    else set_value = 0;
    if(set_value < 10) set_value = 10;  // minimum 10mm, 10 to 1023
    if(set_value > 1000) set_value = 900;  // minimum 10mm, 10 to 1023
    set_value_f = set_value;   //1 to 900
    set_value_f = set_value_f/1000;   //0.01 to 0.9
    set_value_f = min_value * set_value_f;
    set_value = set_value_f;
    
}
//
#separate
void hcsr04_read_1st()
{
	
    //trigger
	output_high(trig);
	delay_us(10);
	output_low(trig);
	//
	while(input(echo)==0) {}
	set_timer1(0);
	while(input(echo)==1)
    {
        //
        if(get_timer1() > 40000)    // us/58=cm, 680cm
        {
            error_flag = 1;
            break;
        }
        else error_flag = 0;
    }
	numberf = (get_timer1()/2) * coeff;	// gettime1 unit is us, coeff unit is cm, max680
    //
    if(error_flag == 0) numberf = numberf * 10; // 0 to 6800mm
    else 
    {
        numberf = 6800;
        error_flag = 0;
    }
    
    if(value_s[0] >= numberf) value_s[0] = numberf;
	min_value = value_s[0];
    //
}
//
#separate
void hcsr04_read()
{
    // moving average
	for (i=3; i>0; i--)
	{
		value_s[i] = value_s[i-1];
	}
    //trigger
	output_high(trig);
	delay_us(10);
	output_low(trig);
	//
	while(input(echo)==0) {}
	set_timer1(0);
	while(input(echo)==1)
    {
        //
        if(get_timer1() > 40000)    //=680
        {
            error_flag = 1;
            break;
        }
        else error_flag = 0;
    }
	numberf = (get_timer1()/2) * coeff;	// gettime1 unit is us, coeff unit is cm
    //
    if(error_flag == 0) numberf = numberf * 10; // 0 to 6800mm
    else 
    {
        numberf = 6800;
        error_flag = 0;
    }
	// moving average
	value_s[0] = numberf;
    //
	for (i=1; i<4; i++)
	{
		numberf = numberf + value_s[i];
	}
	numberf = numberf / 4;  // mm unit
    //
}
//
#separate
void relay_2time()
{
    output_high(relay);
    delay_ms(500);
    output_low(relay);
    delay_ms(500);
    output_high(relay);
    delay_ms(500);
    output_low(relay);
    delay_ms(500);
}
